{ *** TIniPlus ***

by Arash Ramin - http://www.digitalroom.net
version 1.12 (February 19, 2000)

(see readme.txt for details)

}

unit iniplus;

// ReadFloat/WriteFloat missing from Delphi 2/3 and BCB 1
{$IFDEF VER90}
{$DEFINE NEED_FLOAT}
{$ENDIF}

{$IFDEF VER93}
{$DEFINE NEED_FLOAT}
{$ENDIF}

{$IFDEF VER100}
{$DEFINE NEED_FLOAT}
{$ENDIF}

interface

uses inifiles, SysUtils, Graphics, Controls;

type
  TIniPlus = class(TIniFile)
  private

  protected
    {returns a string that represents the Font value}
    function FontToString(Font: TFont) : String;
    {loads the corresponding string settings into Font}
    procedure StringToFont(Str: String; Font: TFont);

    {returns a string that represents the window's size/position}
    function WinPosToString(Win: TControl) : String;
    {loads the corresponding string settings into Win}
    procedure StringToWinPos(Str: String; Win: TControl);

  public
    constructor Create(const FileName: string);
    destructor Destroy; override;

    {$IFDEF NEED_FLOAT}
    function ReadFloat(const Section, Ident: String; Default: Double) : Double;
    procedure WriteFloat(const Section, Ident: String; Value: Double);
    {$ENDIF}

    function ReadColor(const Section, Ident: String; Default: TColor) : TColor;
    procedure WriteColor(const Section, Ident: String; Value: TColor);

    {for ReadFont, the font settings are loaded directly into the Font parameter.
    There is no default setting - in such a case the font is unaltered}
    procedure ReadFont(const Section, Ident: String; Font: TFont);
    procedure WriteFont(const Section, Ident: String; Value: TFont);

    {for ReadWinPos, the size/pos. settings are loaded directly to the control.
    There is no default setting - in such a case the size/pos. is unaltered}
    procedure ReadWinPos(const Section, Ident: String; Win: TControl);
    procedure WriteWinPos(const Section, Ident: String; Win: TControl);

  end;

implementation

{protected}

function TIniPlus.FontToString(Font: TFont) : String;
begin
  // name, size, bold, italic, underline, strikethrough, colour
  Result := Format('%s,%d,%d%d%d%d,%s', [Font.Name, Font.Size,
    Integer(fsBold in Font.Style), Integer(fsItalic in Font.Style),
    Integer(fsUnderline in Font.Style), Integer(fsStrikeOut in Font.Style),
    ColorToString(Font.Color)]);
end;

procedure TIniPlus.StringToFont(Str: String; Font: TFont);
const
  SEP = ',';
  EXCEPT_MSG = 'Invalid string to font conversion';
var
  i: Integer;
begin
  {any exception/error we encounter will ultimately
  result in an EConvertError being raised}
  // name
  i := Pos(SEP, Str);
  if i = 0 then raise EConvertError.Create(EXCEPT_MSG);
  Font.Name := Copy(Str, 1, i-1);
  Delete(Str, 1, i);

  // size
  i := Pos(SEP, Str);
  if i = 0 then raise EConvertError.Create(EXCEPT_MSG);
  Font.Size := StrToInt(Copy(Str, 1, i-1));
  Delete(Str, 1, i);

  // bold, italic, underline, strikethrough
  if Pos(SEP, Str) <> 5 then raise EConvertError.Create(EXCEPT_MSG);
  Font.Style := [];
  if Str[1] = '1' then
    Font.Style := Font.Style + [fsBold];
  if Str[2] = '1' then
      Font.Style := Font.Style + [fsItalic];
  if Str[3] = '1' then
    Font.Style := Font.Style + [fsUnderline];
  if Str[4] = '1' then
    Font.Style := Font.Style + [fsStrikeOut];

  Delete(Str, 1, 5);

  // colour
  Font.Color := StringToColor(Str);
end;

function TIniPlus.WinPosToString(Win: TControl) : String;
begin
  // left, top, width, height
  Result := Format('%d,%d,%d,%d', [Win.Left, Win.Top, Win.Width, Win.Height]);
end;

procedure TIniPlus.StringToWinPos(Str: String; Win: TControl);
const
  SEP = ',';
  EXCEPT_MSG = 'Invalid string to winpos conversion';
var
  i: Integer;
begin
  {any exception/error we encounter will ultimately
  result in an EConvertError being raised}
  // left
  i := Pos(SEP, Str);
  if i = 0 then raise EConvertError.Create(EXCEPT_MSG);
  Win.Left := StrToInt(Copy(Str, 1, i-1));
  Delete(Str, 1, i);

  // top
  i := Pos(SEP, Str);
  if i = 0 then raise EConvertError.Create(EXCEPT_MSG);
  Win.Top := StrToInt(Copy(Str, 1, i-1));
  Delete(Str, 1, i);

  // width
  i := Pos(SEP, Str);
  if i = 0 then raise EConvertError.Create(EXCEPT_MSG);
  Win.Width := StrToInt(Copy(Str, 1, i-1));
  Delete(Str, 1, i);

  // height
  Win.Height := StrToInt(Str);
end;

{public}

constructor TIniPlus.Create(const FileName: string);
begin
  inherited Create(FileName);
end;

destructor TIniPlus.Destroy;
begin
  inherited Destroy;
end;

{$IFDEF NEED_FLOAT}

function TIniPlus.ReadFloat(const Section, Ident: String;
  Default: Double) : Double;
var
  Str: String;
begin
  Str := ReadString(Section, Ident, FloatToStr(Default));
  try
    Result := StrToFloat(Str);
  except
    on EConvertError do Result := Default;
    else raise;
  end;
end;

procedure TIniPlus.WriteFloat(const Section, Ident: String; Value: Double);
begin
  WriteString(Section, Ident, FloatToStr(Value));
end;

{$ENDIF}


function TIniPlus.ReadColor(const Section, Ident: String; Default: TColor)
  : TColor;
var
  Str: String;
begin
  Str := ReadString(Section, Ident, ColorToString(Default));
  try
    Result := StringToColor(Str);
  except
    on EConvertError do Result := Default;
    else raise;
  end;
end;

procedure TIniPlus.WriteColor(const Section, Ident: String; Value: TColor);
begin
  WriteString(Section, Ident, ColorToString(Value));
end;

procedure TIniPlus.ReadFont(const Section, Ident: String; Font: TFont);
var
  DefaultStr, Str: String;
begin
  DefaultStr := FontToString(Font);
  Str := ReadString(Section, Ident, DefaultStr);
  try
    StringToFont(Str, Font);
  except
    on EConvertError do StringToFont(DefaultStr, Font);
    else raise;
  end;
end;

procedure TIniPlus.WriteFont(const Section, Ident: String; Value: TFont);
begin
  WriteString(Section, Ident, FontToString(Value));
end;

procedure TIniPlus.ReadWinPos(const Section, Ident: String; Win: TControl);
var
  DefaultStr, Str: String;
begin
  DefaultStr := WinPosToString(Win);
  Str := ReadString(Section, Ident, DefaultStr);
  try
    StringToWinPos(Str, Win);
  except
    on EConvertError do StringToWinPos(DefaultStr, Win);
    else raise;
  end;
end;

procedure TIniPlus.WriteWinPos(const Section, Ident: String; Win: TControl);
begin
  WriteString(Section, Ident, WinPosToString(Win));
end;

end.
